/****************************************************************************
* settings.cpp
*
* Copyright (C) 2002 Latchesar Ionkov <lionkov@yahoo.com>
* Copyright (C) 2004 Rafal Mantiuk <rafm@users.sourceforge.org>
*
* This program is based on the kbedic dictionary by 
* Radostin Radnev <radnev@yahoo.com>.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
****************************************************************************/

#include <stdio.h>

#include <qlabel.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qspinbox.h>
#include <qcheckbox.h>
#include <qlineedit.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qdialog.h>
#include <qtabdialog.h>
#include <qlistview.h>
#include <qtoolbutton.h>
#include <qvgroupbox.h>
#include <qapplication.h>
#include <qwidgetstack.h>
#include <qtextbrowser.h>

#include <qpe/global.h>
#include <qpe/applnk.h>
#include <qpe/resource.h>
#include <qtopia/qcopenvelope_qws.h>

#include "settings.h"
#include "zbedic.h"

#include <dictionary.h>

#include <assert.h>


class DictItem: public QCheckListItem
{
  Dictionary *dict;
  
public:
  DictItem( QListView *parent, bool isActive, Dictionary *dict ) :
    QCheckListItem( parent, "", QCheckListItem::CheckBox ),
    dict( dict )
    {
      assert( dict != NULL );

      updateStatus( );
      if( !isError() )
        setOn( isActive );
      
    }

  void updateStatus( )
  {
    if( dict->getError() != "" ) {
      QString fname = dict->getFileName().c_str();
      int n = fname.findRev('/');
      QString file = fname.mid(n+1);
      setText( 0, file );
      setEnabled( false );
    } else {
      setText( 0, dict->getName().c_str() );
    }
  }
  
  const char *getFileName() const
  {
    return dict->getFileName().c_str();
  }  

  Dictionary *getDict() 
  {
    return dict;
  }

  bool isActive() const
  {
    return isOn();
  }

  bool isError() const
  {
    return !isEnabled();
  }
  
  
};


ZBEDicSettings::ZBEDicSettings( QWidget *parent ): QTabDialog( parent, "options", true )
{
  setCaption( tr( "Settings" ) );

}

void ZBEDic::settings() {
  QHBox* hbox;
  QLabel* label;

  ZBEDicSettings settings(this);
//  settings.setSpacing(0);
//  settings.setMargin(0);
  settings.setMinimumWidth( 200 );

  // ========= Appearance Tab ==========  
  QVBox* appearanceTab = new QVBox(&settings);
  appearanceTab->setSpacing(4);
  appearanceTab->setMargin(4);
//  appearanceTab->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum, false));

  hbox = new QHBox(appearanceTab);
  hbox->setSpacing(4);
  label = new QLabel(tr("Layout"), hbox);
  QComboBox* layoutModeCB = new QComboBox(FALSE, hbox);
  layoutModeCB->insertItem( "Auto" );
  layoutModeCB->insertItem( "Vertical" );
  layoutModeCB->insertItem( "Horizontal" );
  layoutModeCB->setCurrentItem( layoutMode );

  hbox = new QHBox(appearanceTab);
  hbox->setSpacing(4);
  label = new QLabel(tr("List box size"), hbox);
  QComboBox* listSize = new QComboBox(FALSE, hbox);
  listSize->insertItem( "Small" );
  listSize->insertItem( "Medium" );
  listSize->insertItem( "Large" );
  listSize->setCurrentItem( wordListSize );
  
  QCheckBox *toolBarCB[TB_count];
  QGroupBox *gbox = new QVGroupBox( tr( "Tool bar" ), appearanceTab );
  for( int i = 0; i < TB_count; i++ ) {
    if( (i % 3) == 0 ) {
      hbox = new QHBox(gbox);
    }
    toolBarCB[i] = new QCheckBox( hbox );
    toolBarCB[i]->setPixmap( toolButton[i]->iconSet().pixmap() );
    toolBarCB[i]->setChecked( toolButton[i]->isVisible() );
  }
  
  hbox = new QHBox(appearanceTab);
  hbox->setSpacing(4);
  label = new QLabel(tr("Font"), hbox);
  settings.flist = new QComboBox(FALSE, hbox);
  QStringList fonts = getFontNames();
  int n = 0;
  for(unsigned int i = 0; i < fonts.count(); i++) {
    settings.flist->insertItem(fonts[i]);
    if (fonts[i] == fontName) {
      n = i;
    }
  }
  settings.flist->setCurrentItem(n);
  settings.fsize = new QSpinBox(4, 32, 1, hbox);
  settings.fsize->setValue(fontSize);

  hbox = new QHBox(appearanceTab);
  settings.fontPreviewLB = new QLabel(tr("ABCDEF abcdef 12345"), hbox);
  settings.fontPreviewLB->setFrameStyle( QFrame::Box | QFrame::Sunken );

  connect( settings.flist, SIGNAL(activated(int)), 
    &settings, SLOT(updateFontPreview()) );
  connect( settings.fsize, SIGNAL(valueChanged(int)), 
    &settings, SLOT(updateFontPreview()) );

  settings.updateFontPreview();
  
  hbox = new QHBox(appearanceTab);
  appearanceTab->setStretchFactor( hbox, 1 );
  
  settings.addTab( appearanceTab, tr("Layout") );

  // ========= Dictionaries Tab ==========
  QVBox* dictTab = new QVBox(&settings); 
  dictTab->setSpacing(4);
  dictTab->setMargin(4);
//  dictTab->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum, false));
  
  settings.dictList = new QListView( dictTab );
  settings.dictList->addColumn( tr( "Dictionary" ) );
//  settings.dictList->setMinimumHeight( fontHeight*8 );

  for( uint i = 0; i < dictionaries.count(); i++ )
    new DictItem( settings.dictList, true, dictionaries.at(i) );
  
  settings.dictFileLB = new QLabel( QString::null, dictTab );

  hbox = new QHBox(dictTab);  
  settings.dictActiveLB = new QLabel( QString::null, hbox );
  QPushButton *dictInfoBT = new QPushButton( tr( "Details" ), hbox );
  dictInfoBT->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum, false));
  connect( dictInfoBT, SIGNAL(clicked()), 
    &settings, SLOT(showDictInfo()));

  settings.searchDictWS = new QWidgetStack( dictTab );
  
  settings.dictSearchBt = new QPushButton( tr( "Search for new dictionaries" ), settings.searchDictWS );
  settings.searchDictWS->addWidget( settings.dictSearchBt, 1 );
  settings.dictSearchLB = new QLabel( QString::null, settings.searchDictWS );
  settings.dictSearchLB->setFrameStyle( QFrame::Panel + QFrame::Sunken );
  settings.searchDictWS->addWidget( settings.dictSearchLB, 2 );
  settings.searchDictWS->raiseWidget( settings.dictSearchBt );
//  settings.searchDictWS->setMinimumHeight( settings.dictSearchBt->minimumHeight() );
  // Seems to be bug in QtEmbeeded - QWidgetStack has wrong size policy
  settings.searchDictWS->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum, false));
  
//  dictSearchBt->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum, false));
  connect( settings.dictSearchBt, SIGNAL(clicked()), 
    &settings, SLOT(searchForDictionaries()));
  connect( settings.dictList, SIGNAL(currentChanged(QListViewItem*)), 
    &settings, SLOT(dictSelected(QListViewItem*)));
  connect( settings.dictList, SIGNAL(clicked(QListViewItem*)), 
    &settings, SLOT(dictSelected(QListViewItem*)));
  connect( settings.dictList, SIGNAL(returnPressed(QListViewItem*)), 
    &settings, SLOT(toggleActive(QListViewItem*)));
  
//   if( settings.dictList->firstChild() != NULL ) {
//     settings.dictList->setCurrentItem(settings.dictList->firstChild());
//   }  
  
  settings.addTab( dictTab, tr("Dictionaries") );
  
  // ========= Advanced Tab ==========
  QVBox* advancedTab = new QVBox(&settings);
  advancedTab->setSpacing(4);
  advancedTab->setMargin(4);
//  advancedTab->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum, false));
  
  hbox = new QHBox(advancedTab);
  hbox->setSpacing(4);
  
  label = new QLabel(tr("Words in list"), hbox);
  QSpinBox* llen = new QSpinBox(8, 500, 1, hbox);
  llen->setValue(listLen);
        
  hbox = new QHBox(advancedTab);
  hbox->setSpacing(4);
  label = new QLabel(tr("History length"), hbox);
  QSpinBox* hlen = new QSpinBox(8, 100, 1, hbox);
  hlen->setValue(historyLen);

  QPushButton *clearHistBt = new QPushButton( tr( "Clear history" ), advancedTab );
//  dictSearchBt->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum, false));
  connect( clearHistBt, SIGNAL(clicked()), 
    this, SLOT(clearHistory()));
  
//   hbox = new QHBox(advancedTab);
//   hbox->setSpacing(4);
//   hbox->setMargin(2);
//   label = new QLabel(tr("Dictionary Path"), hbox);
//   QString dp;
//   for(unsigned int i = 0; i < dictPaths.count(); i++) {
//     dp += dictPaths[i] + ":";
//   }
//   QLineEdit* dpath = new QLineEdit(dp, hbox);
  
  QCheckBox* spron = new QCheckBox(tr("Show pronounciation"), advancedTab);
  spron->setChecked(showPron);

  hbox = new QHBox(advancedTab);
  advancedTab->setStretchFactor( hbox, 2 );
  
  settings.addTab( advancedTab, tr("Misc") );
  
//  appearanceTab->adjustSize();
  settings.adjustSize();
  settings.setOkButton( QString() );

  if (settings.exec() == QDialog::Accepted) {
    listLen = llen->value();
    historyLen = hlen->value();
//     dictPaths=QStringList::split(':', dpath->text());

    for( int i = 0; i < TB_count; i++ ) {      
      if( toolBarCB[i]->isChecked() )
        toolButton[i]->show();
      else
        toolButton[i]->hide();
    }
    
    QString fn = settings.flist->currentText();
    int fs = settings.fsize->value();
    if (fn!=fontName || fs!=fontSize) {
      fontName = fn;
      fontSize = fs;
      changeFont(fn, fs);
    }

    showPron = spron->isChecked();
    descrFormatter->setShowPron(showPron);
                
    layoutMode = layoutModeCB->currentItem();
    wordListSize = listSize->currentItem();
    updateLayout( wordListSize, layoutMode );
    mainLayout->activate();

    bool currentInactive = false;
    dictionaries.clear();
    for( QListViewItem *item = settings.dictList->firstChild(); item != NULL; item = item->nextSibling() ) {
      DictItem *dictItem = (DictItem*)item;
      if( dictItem->isActive() ) {
        dictionaries.inSort(dictItem->getDict());
      } else {
        if( dictItem->getDict() == dict ) {
          currentInactive = true;
          dict = NULL;
        }
        delete dictItem->getDict();
      }
    }

    fillInDictPopup();

    if( currentInactive || dict == NULL ) { // If currect dict is inactive or no dict slected
      Dictionary *d = NULL;
      if (dictionaries.count() > 0) {
        d = dictionaries.at(0);
      }
      setDictionary( d );
    }
    
  } else {
    // Delete loaded, but inactiv dictionaries
    for( QListViewItem *item = settings.dictList->firstChild(); item != NULL; item = item->nextSibling() ) {
      DictItem *dictItem = (DictItem*)item;
      if( dictionaries.find( dictItem->getDict() ) == -1 )
        delete dictItem->getDict();
    }
  }

}

void ZBEDicSettings::searchForDictionaries()
{
//  QCopEnvelope busyStart( "QPE/System", "busy()" );
//  qApp->processEvents();
  searchDictWS->raiseWidget( dictSearchLB );
  dictSearchLB->setText( tr( "Searching for dictionaries..." ) );
  qApp->processEvents();
    
  DocLnkSet fset;
  Global::findDocuments(&fset, "application/bedic");
  
  QList<DocLnk> flist = fset.children();
  
  for(unsigned int ii = 0; ii < flist.count(); ii++) {
    const DocLnk* f = flist.at(ii);

    QString fname = f->file();

    bool itemExist = false;
    for( QListViewItem *item = dictList->firstChild(); item != NULL; item = item->nextSibling() ) {
      DictItem *dictItem = (DictItem*)item;
      if( dictItem->getFileName() == fname ) {
        itemExist = true;
        break;
      }
    }
    
    if( !itemExist ) {
      Dictionary* d = Dictionary::create(fname, false);
      string errorMessage = d->getError();
      if( errorMessage != "" )
        delete d;
      else {
        DictItem *item = new DictItem( dictList, false, d );
        dictList->insertItem( item );
      }
    } 
  }

  dictSearchLB->setText( tr( "Checking integrity..." ) );
  qApp->processEvents();
  
  for( QListViewItem *item = dictList->firstChild(); item != NULL; item = item->nextSibling() ) {
    DictItem *dictItem = (DictItem*)item;
    dictItem->getDict()->checkIntegrity();
    dictItem->updateStatus();
  }

  searchDictWS->raiseWidget( dictSearchBt );
//  qApp->processEvents();
  
//  QCopEnvelope busyEnd( "QPE/System", "notBusy()" );
}

void ZBEDicSettings::dictSelected( QListViewItem *item )
{
  if( item == NULL ) return;
  
  DictItem *ditem = (DictItem*)item;
  dictFileLB->setText( ditem->getFileName() );

  QString statusText = tr( "Status: " );
  if( ditem->isError() ) {
    statusText += ditem->getDict()->getError().c_str();
  } else {
    statusText += ditem->isActive() ? tr( "Active" ) : tr( "Inactive" );
  }
  dictActiveLB->setText( statusText );
}

void ZBEDicSettings::toggleActive( QListViewItem *item )
{
  QCheckListItem *citem = (QCheckListItem*)item;
  if( citem->isEnabled() )
    citem->setOn( !citem->isOn() );
  dictSelected( item );
}

void ZBEDicSettings::updateFontPreview()
{
  QString fn = flist->currentText();
  int fs = fsize->value();

  fontPreviewLB->setFont( QFont( fn, fs ) );
}

// class needed to handle resize events
class DictInfoDialog: public QDialog
{
  QTextBrowser *info;
  QVBox* main;
public:
  DictInfoDialog( QWidget *parent ) :
    QDialog( parent, "Dict Info dialog", true )
  {
    main = new QVBox(this);
    main->setSpacing(4);
    main->setMargin(2);
//    main->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding, false));
    info = new QTextBrowser( main );
//    info->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding, false));
//    setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding, false));
//    setSizePolicy(QSizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum, false));
  }

  void setInfoText( QString &text )
  {
    info->setText( text );
    updateGeometry();
    main->setMinimumSize( size() );
    main->setMaximumSize( size() );    
    main->adjustSize();
  }
  

protected:
//   void resizeEvent ( QResizeEvent *e )
//   {
//     QDialog::resizeEvent( e );
//     main->setMaximumSize( e->size() );
//     main->adjustSize();
//   }
  
};

static QString escapeString( const string &str )
{
  QString s = QString::fromUtf8( str.c_str() );
  s = s.replace( QRegExp("<"), "&lt;" );
  s = s.replace( QRegExp(">"), "&gt;" );
  return s;
}

void ZBEDicSettings::showDictInfo()
{
  DictItem *ditem = (DictItem*)dictList->selectedItem();
  if( ditem == NULL ) return;
  Dictionary *d = ditem->getDict();

  DictInfoDialog dictInfoDialog( this ); //, "Dict Info dialog", true );
  dictInfoDialog.showMaximized();
  
  QString infoString;
  infoString.append( "Dictionary: <b>" );
  infoString.append( d->getName().c_str() );
  infoString.append( "</b><br>" );
  infoString.append( "File: <i>" );
  infoString.append( d->getFileName().c_str() );
  infoString.append( "</i><br><hr>" );

  string maintainer = d->getProperty( "maintainer" );
  string copyright = d->getProperty( "copyright" );
  string description = d->getProperty( "description" );

  if( copyright.size() != 0 ) {
    infoString.append( escapeString( copyright ) );  
    infoString.append( "<br>" );  
  }
  
  if( description.size() != 0 ) {
    infoString.append( escapeString( description ) );  
    infoString.append( "<br>" );  
  }

  if( maintainer.size() != 0 ) {
    infoString.append( "<i>Report any problems with the dictionary to " );
    infoString.append( escapeString( maintainer ) );  
    infoString.append( "</i><br>" );  
  }  
  
  // Show comment lines
  for( int i = 0; i < 20; i++ ) {
    char cname[10];
    sprintf( cname, "comment%02d", i );
    string commentLine = d->getProperty( cname );
    if( commentLine.size() == 0 ) break;
    infoString.append( escapeString( commentLine ) + " " );    
  }

  string items = d->getProperty( "items" );  
  string builddate = d->getProperty( "builddate" );
  string dicsize = d->getProperty( "dict-size" );
  if( items.size() != 0 || builddate.size() != 0 || dicsize.size() != 0 ) {
    infoString.append( "<hr>Items in dictionary: " );  
    infoString.append( items.c_str() );  
    infoString.append( "<br>Build date: <i>" );  
    infoString.append( builddate.c_str() );  
    infoString.append( "</i><br>Size (uncompressed): " );  
    infoString.append( dicsize.c_str() );  
    infoString.append( " bytes</br>" );  
  }
  
  dictInfoDialog.setCaption( d->getName().c_str() );  

//  dictInfoDialog.setSizeGripEnabled( TRUE );
  
  dictInfoDialog.setInfoText( infoString );
//  dictInfoDialog.adjustSize();
  dictInfoDialog.exec();
}

void ZBEDic::clearHistory()
{
  wordBox->listBox()->clear();
  wordBox->listBox()->setCurrentItem( -1 );
  updateNavigationButtons();
}
